# Copyright (c) 2016, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase

from stix.core import STIXPackage
from stix.extensions.marking import ais


class AISMarkingStructureNotProprietaryTests(EntityTestCase, unittest.TestCase):
    klass = ais.AISMarkingStructure
    _full_dict = {
        'not_proprietary':
            {
                'cisa_proprietary': 'false',
                'ais_consent': {'consent': 'NONE'},
                'tlp_marking': {'color': 'GREEN'}
            },
        'xsi:type': 'AIS:AISMarkingStructure'
    }


class AISMarkingStructureIsProprietaryTests(EntityTestCase, unittest.TestCase):
    klass = ais.AISMarkingStructure
    _full_dict = {
        'is_proprietary':
            {
                'cisa_proprietary': 'true',
                'ais_consent': {'consent': 'EVERYONE'},
                'tlp_marking': {'color': 'AMBER'}
            },
        'xsi:type': 'AIS:AISMarkingStructure'
    }


class AISMarkingGeneralTests(unittest.TestCase):
    """General tests for ais module helpers."""

    @classmethod
    def setUpClass(cls):
        cls.INDUSTRY_SECTORS = (ais.CHEMICAL_SECTOR, ais.COMMERCIAL_FACILITIES_SECTOR,
                                ais.COMMUNICATIONS_SECTOR,
                                ais.CRITICAL_MANUFACTURING_SECTOR,
                                ais.DAMS_SECTOR, ais.DEFENSE_INDUSTRIAL_BASE_SECTOR,
                                ais.EMERGENCY_SERVICES_SECTOR, ais.ENERGY_SECTOR,
                                ais.FINANCIAL_SERVICES_SECTOR,
                                ais.FOOD_AND_AGRICULTURE_SECTOR,
                                ais.GOVERNMENT_FACILITIES_SECTOR,
                                ais.HEALTH_CARE_AND_PUBLIC_HEALTH_SECTOR,
                                ais.INFORMATION_TECHNOLOGY_SECTOR,
                                ais.NUCLEAR_REACTORS_MATERIALS_AND_WASTE_SECTOR,
                                ais.TRANSPORTATION_SYSTEMS_SECTOR, ais.OTHER,
                                ais.WATER_AND_WASTEWATER_SYSTEMS_SECTOR)

    def test_validate_and_create_industry_type(self):
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, [])
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, "")
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, [""])
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, ["Energy Sector", "Others", "Dams Sector"])
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, "Energy Sector| Others|Dams Sector")
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, 3)
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, "|")
        self.assertRaises(ValueError, ais._validate_and_create_industry_type, ["Energy Sector|Dams Sector"])

        self.assertEqual(ais._validate_and_create_industry_type("eNergY sectOr"), ais.ENERGY_SECTOR)
        self.assertEqual(ais._validate_and_create_industry_type("eNergY sectOr   |Dams sector"), "Energy Sector|Dams Sector")
        self.assertEqual(ais._validate_and_create_industry_type(["eNergY sectOr    ", "    Dams sectOr    "]), "Energy Sector|Dams Sector")
        self.assertEqual(ais._validate_and_create_industry_type([ais.ENERGY_SECTOR, ais.DAMS_SECTOR]), "Energy Sector|Dams Sector")
        self.assertEqual(ais._validate_and_create_industry_type("Energy Sector|Dams Sector"), "Energy Sector|Dams Sector")

        for idx, x in enumerate(self.INDUSTRY_SECTORS):
            self.assertEqual(ais._validate_and_create_industry_type(x), self.INDUSTRY_SECTORS[idx])

    def test_add_ais_marking(self):
        PACKAGE = STIXPackage()

        # Missing kwarg.
        self.assertRaises(ValueError, ais.add_ais_marking, PACKAGE, True, 'NONE', 'GREEN',
                        country_name_code='US',
                        country_name_code_type='ISO 3166-1 alpha-2',
                        admin_area_name_code='US-DC',
                        admin_area_name_code_type='ISO 3166-2',
                        organisation_name='NCCIC')

    def test_bad_values_raises_error(self):
        self.assertRaises(ValueError, ais.TLPMarkingType, "ORANGE")
        self.assertRaises(ValueError, ais.AISConsentType, "WHAT?")

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Generated Thu Apr 11 15:07:57 2013 by generateDS.py version 2.9a.
#

import sys

from mixbox.binding_utils import *

from stix.bindings import register_extension
import stix.bindings.indicator as indicator_binding
import stix.bindings.stix_common as stix_common_binding

XML_NS = "http://stix.mitre.org/extensions/TestMechanism#Snort-1"

#
# Data representation classes.
#

@register_extension
class SnortTestMechanismType(indicator_binding.TestMechanismType):
    """The SnortTestMechanismType specifies an instantial extension from
    the abstract TestMechanismType intended to support the inclusion
    of a Snort rule as a test mechanism content."""
    subclass = None
    superclass = indicator_binding.TestMechanismType

    xmlns          = XML_NS
    xmlns_prefix   = "snortTM"
    xml_type       = "SnortTestMechanismType"
    xsi_type       = "%s:%s" % (xmlns_prefix, xml_type)

    def __init__(self, idref=None, id=None, Efficacy=None, Producer=None, Product_Name=None, Version=None, Rule=None, Event_Filter=None, Rate_Filter=None, Event_Suppression=None):
        super(SnortTestMechanismType, self).__init__(idref=idref, id=id, Efficacy=Efficacy, Producer=Producer)
        self.Product_Name = Product_Name
        self.Version = Version
        if Rule is None:
            self.Rule = []
        else:
            self.Rule = Rule
        if Event_Filter is None:
            self.Event_Filter = []
        else:
            self.Event_Filter = Event_Filter
        if Rate_Filter is None:
            self.Rate_Filter = []
        else:
            self.Rate_Filter = Rate_Filter
        if Event_Suppression is None:
            self.Event_Suppression = []
        else:
            self.Event_Suppression = Event_Suppression
    def factory(*args_, **kwargs_):
        if SnortTestMechanismType.subclass:
            return SnortTestMechanismType.subclass(*args_, **kwargs_)
        else:
            return SnortTestMechanismType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Product_Name(self): return self.Product_Name
    def set_Product_Name(self, Product_Name): self.Product_Name = Product_Name
    def get_Version(self): return self.Version
    def set_Version(self, Version): self.Version = Version
    def get_Rule(self): return self.Rule
    def set_Rule(self, Rule): self.Rule = Rule
    def add_Rule(self, value): self.Rule.append(value)
    def insert_Rule(self, index, value): self.Rule[index] = value
    def get_Event_Filter(self): return self.Event_Filter
    def set_Event_Filter(self, Event_Filter): self.Event_Filter = Event_Filter
    def add_Event_Filter(self, value): self.Event_Filter.append(value)
    def insert_Event_Filter(self, index, value): self.Event_Filter[index] = value
    def get_Rate_Filter(self): return self.Rate_Filter
    def set_Rate_Filter(self, Rate_Filter): self.Rate_Filter = Rate_Filter
    def add_Rate_Filter(self, value): self.Rate_Filter.append(value)
    def insert_Rate_Filter(self, index, value): self.Rate_Filter[index] = value
    def get_Event_Suppression(self): return self.Event_Suppression
    def set_Event_Suppression(self, Event_Suppression): self.Event_Suppression = Event_Suppression
    def add_Event_Suppression(self, value): self.Event_Suppression.append(value)
    def insert_Event_Suppression(self, index, value): self.Event_Suppression[index] = value
    def hasContent_(self):
        if (
            self.Product_Name is not None or
            self.Version is not None or
            self.Rule or
            self.Event_Filter or
            self.Rate_Filter or
            self.Event_Suppression or
            super(SnortTestMechanismType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='SnortTestMechanismType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='SnortTestMechanismType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='snortTM:', name_='SnortTestMechanismType'):
        super(SnortTestMechanismType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='SnortTestMechanismType')
        # if 'xmlns' not in already_processed:
        #     already_processed.add('xmlns')
        #     xmlns = " xmlns:%s='%s'" % (self.xmlns_prefix, self.xmlns)
        #     lwrite(xmlns)
        if 'xsi:type' not in already_processed:
            already_processed.add('xsi:type')
            xsi_type = " xsi:type='%s:%s'" % (self.xmlns_prefix, self.xml_type)
            lwrite(xsi_type)
    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='SnortTestMechanismType', fromsubclass_=False, pretty_print=True):
        super(SnortTestMechanismType, self).exportChildren(lwrite, level, nsmap, indicator_binding.XML_NS, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Product_Name is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%s:Product_Name>%s</%s:Product_Name>%s' % (nsmap[namespace_], quote_xml(self.Product_Name), nsmap[namespace_], eol_))
        if self.Version is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%s:Version>%s</%s:Version>%s' % (nsmap[namespace_], quote_xml(self.Version), nsmap[namespace_], eol_))
        for Rule_ in self.Rule:
            Rule_.export(lwrite, level, nsmap, namespace_, name_='Rule', pretty_print=pretty_print)
        for Event_Filter_ in self.Event_Filter:
            Event_Filter_.export(lwrite, level, nsmap, namespace_, name_='Event_Filter', pretty_print=pretty_print)
        for Rate_Filter_ in self.Rate_Filter:
            Rate_Filter_.export(lwrite, level, nsmap, namespace_, name_='Rate_Filter', pretty_print=pretty_print)
        for Event_Suppression_ in self.Event_Suppression:
            Event_Suppression_.export(lwrite, level, nsmap, namespace_, name_='Event_Suppression', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(SnortTestMechanismType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Product_Name':
            Product_Name_ = child_.text
            Product_Name_ = self.gds_validate_string(Product_Name_, node, 'Product_Name')
            self.Product_Name = Product_Name_
        elif nodeName_ == 'Version':
            Version_ = child_.text
            Version_ = self.gds_validate_string(Version_, node, 'Version')
            self.Version = Version_
        elif nodeName_ == 'Rule':
            obj_ = stix_common_binding.EncodedCDATAType.factory()
            obj_.build(child_)
            self.Rule.append(obj_)
        elif nodeName_ == 'Event_Filter':
            obj_ = stix_common_binding.EncodedCDATAType.factory()
            obj_.build(child_)
            self.Event_Filter.append(obj_)
        elif nodeName_ == 'Rate_Filter':
            obj_ = stix_common_binding.EncodedCDATAType.factory()
            obj_.build(child_)
            self.Rate_Filter.append(obj_)
        elif nodeName_ == 'Event_Suppression':
            obj_ = stix_common_binding.EncodedCDATAType.factory()
            obj_.build(child_)
            self.Event_Suppression.append(obj_)
        super(SnortTestMechanismType, self).buildChildren(child_, node, nodeName_, True)
# end class SnortTestMechanismType




GDSClassesMapping = {}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'SnortTestMechanismType'
        rootClass = SnortTestMechanismType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout, 0, name_=rootTag,
        namespacedef_='',
        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'SnortTestMechanismType'
        rootClass = SnortTestMechanismType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'SnortTestMechanismType'
        rootClass = SnortTestMechanismType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    # sys.stdout.write('<?xml version="1.0" ?>\n')
    # rootObj.export(sys.stdout, 0, name_="Indicator",
    #     namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "SnortTestMechanismType"
    ]



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""
STIX Extension for AIS Data Markings

Unlike the other marking extensions, the AIS marking extension is not loaded
automatically, since AIS markings are not a part of the published STIX 1.x
specifications. They are included in python-stix because they're common enough
that it is not worth creating a separate package.

If you are writing code that needs to parse AIS markings, make sure that your
program imports this module before beginning to parse any STIX documents:

.. code-block:: python

    import stix.extensions.marking.ais

"""

from mixbox import fields
from mixbox.namespaces import Namespace

import stix.bindings.extensions.marking.ais as ais_binding
import stix.data_marking
from stix.data_marking import MarkingStructure


def validate_value(instance, value):
    allowed = instance._ALLOWED_VALUES
    if not value:
        return
    elif not allowed:
        return
    elif value in allowed:
        return
    else:
        error = "Value must be one of {allowed}. Received '{value}'"
        error = error.format(**locals())
        raise ValueError(error)


class AISConsentType(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.AISConsentType
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'
    _ALLOWED_VALUES = ('EVERYONE', 'USG', 'NONE')

    consent = fields.TypedField("consent", preset_hook=validate_value)

    def __init__(self, consent=None):
        super(AISConsentType, self).__init__()
        self.consent = consent


class TLPMarkingType(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.TLPMarkingType
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'
    _ALLOWED_VALUES = ('WHITE', 'GREEN', 'AMBER')

    color = fields.TypedField("color", preset_hook=validate_value)

    def __init__(self, color=None):
        super(TLPMarkingType, self).__init__()
        self.color = color


class NotProprietary(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.NotProprietary
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'

    cisa_proprietary = fields.TypedField("CISA_Proprietary")
    ais_consent = fields.TypedField("AISConsent", AISConsentType, key_name="ais_consent")
    tlp_marking = fields.TypedField("TLPMarking", TLPMarkingType, key_name="tlp_marking")

    def __init__(self, cisa_proprietary='false', ais_consent=None,
                 tlp_marking=None):
        super(NotProprietary, self).__init__()

        self.cisa_proprietary = cisa_proprietary
        self.ais_consent = ais_consent
        self.tlp_marking = tlp_marking


class IsProprietary(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.IsProprietary
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'

    cisa_proprietary = fields.TypedField("CISA_Proprietary")
    ais_consent = fields.TypedField("AISConsent", AISConsentType, key_name="ais_consent")
    tlp_marking = fields.TypedField("TLPMarking", TLPMarkingType, key_name="tlp_marking")

    def __init__(self, cisa_proprietary='true', ais_consent=None,
                 tlp_marking=None):
        super(IsProprietary, self).__init__()

        self.cisa_proprietary = cisa_proprietary
        self.ais_consent = ais_consent
        self.tlp_marking = tlp_marking


@stix.register_extension
class AISMarkingStructure(MarkingStructure):
    _binding = ais_binding
    _binding_class = _binding.AISMarkingStructure
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'
    _XSI_TYPE = "AIS:AISMarkingStructure"

    is_proprietary = fields.TypedField("Is_Proprietary", IsProprietary)
    not_proprietary = fields.TypedField("Not_Proprietary", NotProprietary)

    def __init__(self, is_proprietary=None, not_proprietary=None):
        super(AISMarkingStructure, self).__init__()

        self.is_proprietary = is_proprietary
        self.not_proprietary = not_proprietary


NAMESPACES = [
    Namespace('http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2', 'AIS', 'http://www.us-cert.gov/sites/default/files/STIX_Namespace/AIS_Bundle_Marking_1.1.1_v1.0.xsd')
]


def _update_namespaces():
    # Update the python-stix namespace dictionary
    from stix.utils import nsparser
    import mixbox.namespaces

    nsparser.STIX_NAMESPACES.add_namespace(NAMESPACES[0])
    mixbox.namespaces.register_namespace(NAMESPACES[0])


_update_namespaces()


# IndustryType allowed sectors
#: Chemical Sector
CHEMICAL_SECTOR = 'Chemical Sector'
#: Chemical Sector
COMMERCIAL_FACILITIES_SECTOR = 'Commercial Facilities Sector'
#: Commercial Facilities Sector
COMMUNICATIONS_SECTOR = 'Communications Sector'
#: Critical Manufacturing Sector
CRITICAL_MANUFACTURING_SECTOR = 'Critical Manufacturing Sector'
#: Dams Sector
DAMS_SECTOR = 'Dams Sector'
#: Defense Industrial Base Sector
DEFENSE_INDUSTRIAL_BASE_SECTOR = 'Defense Industrial Base Sector'
#: Emergency Services Sector
EMERGENCY_SERVICES_SECTOR = 'Emergency Services Sector'
#: Energy Sector
ENERGY_SECTOR = 'Energy Sector'
#: Financial Services Sector
FINANCIAL_SERVICES_SECTOR = 'Financial Services Sector'
#: Food and Agriculture Sector
FOOD_AND_AGRICULTURE_SECTOR = 'Food and Agriculture Sector'
#: Government Facilities Sector
GOVERNMENT_FACILITIES_SECTOR = 'Government Facilities Sector'
#: Healthcare and Public Health Sector
HEALTH_CARE_AND_PUBLIC_HEALTH_SECTOR = 'Healthcare and Public Health Sector'
#: Information Technology Sector
INFORMATION_TECHNOLOGY_SECTOR = 'Information Technology Sector'
#: Nuclear Reactors, Materials, and Waste Sector
NUCLEAR_REACTORS_MATERIALS_AND_WASTE_SECTOR = 'Nuclear Reactors, Materials, and Waste Sector'
#: Other
OTHER = 'Other'
#: Transportation Systems Sector
TRANSPORTATION_SYSTEMS_SECTOR = 'Transportation Systems Sector'
#: Water and Wastewater Systems Sector
WATER_AND_WASTEWATER_SYSTEMS_SECTOR = 'Water and Wastewater Systems Sector'


def _validate_and_create_industry_type(industry_type):
    INDUSTRY_SECTORS = (CHEMICAL_SECTOR, COMMERCIAL_FACILITIES_SECTOR,
                        COMMUNICATIONS_SECTOR, CRITICAL_MANUFACTURING_SECTOR,
                        DAMS_SECTOR, DEFENSE_INDUSTRIAL_BASE_SECTOR,
                        EMERGENCY_SERVICES_SECTOR, ENERGY_SECTOR,
                        FINANCIAL_SERVICES_SECTOR, FOOD_AND_AGRICULTURE_SECTOR,
                        GOVERNMENT_FACILITIES_SECTOR,
                        HEALTH_CARE_AND_PUBLIC_HEALTH_SECTOR,
                        INFORMATION_TECHNOLOGY_SECTOR,
                        NUCLEAR_REACTORS_MATERIALS_AND_WASTE_SECTOR,
                        TRANSPORTATION_SYSTEMS_SECTOR, OTHER,
                        WATER_AND_WASTEWATER_SYSTEMS_SECTOR)

    lower_case_sectors = tuple(x.lower() for x in INDUSTRY_SECTORS)
    result = ""
    error = False
    val = []

    if isinstance(industry_type, str):
        # Pipe-delimited or single string supplied.
        val = [x.lower().strip() for x in industry_type.split("|")]

    elif isinstance(industry_type, (list, tuple)):
        # Create pipe-delimited string when list of strings is provided.
        val = [x.lower().strip() for x in industry_type]

    else:
        error = True

    for item in val:
        for idx, sector in enumerate(lower_case_sectors):
            if item == sector:
                if not result:
                    result = INDUSTRY_SECTORS[idx]
                else:
                    result = "{0}|{1}".format(result, INDUSTRY_SECTORS[idx])
                break
        else:
            # The sectors collection was exhausted. No match found.
            error = True
            break

    if not error and val:
        return result

    msg = 'IndustryType must be one of the following: {0}. Received \'{1}\'.'
    raise ValueError(msg.format(INDUSTRY_SECTORS, industry_type))


def add_ais_marking(stix_package, proprietary, consent, color, **kwargs):
    """
    This utility functions aids in the creation of an AIS marking and appends
    it to the provided STIX package.

    Args:
        stix_package: A stix.core.STIXPackage object.
        proprietary: True if marking uses IsProprietary, False for
            NotProprietary.
        consent: A string with one of the following values: "EVERYONE", "NONE"
            or "USG".
        color: A string that corresponds to TLP values: "WHITE", "GREEN" or
            "AMBER".
        **kwargs: Six required keyword arguments that are used to create a CIQ
            identity object. These are: country_name_code,
            country_name_code_type, admin_area_name_code,
            admin_area_name_code_type, organisation_name, industry_type.

    Raises:
        ValueError: When keyword arguments are missing. User did not supply
            correct values for: proprietary, color and consent.

    Note:
        The following line is required to register the AIS extension::

            >>> import stix.extensions.marking.ais

        Any Markings under STIX Header will be removed. Please follow the
        guidelines for `AIS`_.

        The industry_type keyword argument accepts: a list of string based on
        defined sectors, a pipe-delimited string of sectors, or a single
        sector.

    .. _AIS:
        https://www.us-cert.gov/ais

    """
    from stix.common import InformationSource
    from stix.extensions.identity.ciq_identity_3_0 import (
        CIQIdentity3_0Instance, STIXCIQIdentity3_0, PartyName, Address,
        Country, NameElement, OrganisationInfo, AdministrativeArea)
    from stix.core.stix_header import STIXHeader
    from stix.data_marking import MarkingSpecification, Marking

    args = ('country_name_code', 'country_name_code_type', 'industry_type',
            'admin_area_name_code', 'admin_area_name_code_type',
            'organisation_name')

    diff = set(args) - set(kwargs.keys())

    if diff:
        msg = 'All keyword arguments must be provided. Missing: {0}'
        raise ValueError(msg.format(tuple(diff)))

    party_name = PartyName()
    party_name.add_organisation_name(kwargs['organisation_name'])

    country = Country()
    country_name = NameElement()
    country_name.name_code = kwargs['country_name_code']
    country_name.name_code_type = kwargs['country_name_code_type']
    country.add_name_element(country_name)

    admin_area = AdministrativeArea()
    admin_area_name = NameElement()
    admin_area_name.name_code = kwargs['admin_area_name_code']
    admin_area_name.name_code_type = kwargs['admin_area_name_code_type']
    admin_area.add_name_element(admin_area_name)

    address = Address()
    address.country = country
    address.administrative_area = admin_area

    org_info = OrganisationInfo()
    org_info.industry_type = _validate_and_create_industry_type(kwargs['industry_type'])

    id_spec = STIXCIQIdentity3_0()
    id_spec.party_name = party_name
    id_spec.add_address(address)
    id_spec.organisation_info = org_info

    identity = CIQIdentity3_0Instance()
    identity.specification = id_spec

    if proprietary is True:
        proprietary_obj = IsProprietary()
        consent = 'EVERYONE'
    elif proprietary is False:
        proprietary_obj = NotProprietary()
    else:
        raise ValueError('proprietary expected True or False.')

    proprietary_obj.ais_consent = AISConsentType(consent=consent)
    proprietary_obj.tlp_marking = TLPMarkingType(color=color)

    ais_marking = AISMarkingStructure()

    if isinstance(proprietary_obj, IsProprietary):
        ais_marking.is_proprietary = proprietary_obj
    else:
        ais_marking.not_proprietary = proprietary_obj

    marking_spec = MarkingSpecification()
    marking_spec.controlled_structure = '//node() | //@*'
    marking_spec.marking_structures.append(ais_marking)
    marking_spec.information_source = InformationSource()
    marking_spec.information_source.identity = identity

    if not stix_package.stix_header:
        stix_package.stix_header = STIXHeader()

    # Removes any other Markings if present.
    stix_package.stix_header.handling = Marking()
    stix_package.stix_header.handling.add_marking(marking_spec)



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# external
from mixbox import fields


# internal
import stix
from stix.bindings.extensions.malware import maec_4_1 as maec_instance_binding
from stix.ttp.malware_instance import MalwareInstance


@stix.register_extension
class MAECInstance(MalwareInstance):
    """
    The MAECInstance object provides an extension to the MalwareInstanceType
    which imports and leverages the MAEC 4.1 schema for structured
    characterization of Malware.

    This class extension is automatically registered by the
    MalwareInstanceFactory.

    Warnings:
        Interacting with the ``maec`` field will fail if the maec library is
        not installed in your Python environment.
    """
    _binding = maec_instance_binding
    _binding_class = _binding.MAEC4_1InstanceType
    _namespace = "http://stix.mitre.org/extensions/Malware#MAEC4.1-1"
    _XSI_TYPE = "stix-maec:MAEC4.1InstanceType"

    maec = fields.TypedField("MAEC", type_="maec.package.package.Package")

    def __init__(self, maec=None):
        super(MAECInstance, self).__init__()
        self.maec = maec



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
from functools import partial

# mixbox
from mixbox import fields

# stix
import stix
from stix.ttp import TTP
from stix.common.kill_chains import KillChains
from stix.bindings import stix_core as core_binding

# deprecation warnings
from stix.utils.deprecated import IdrefDeprecatedList


class TTPs(stix.Entity):
    _binding = core_binding
    _binding_class = _binding.TTPsType
    _namespace = 'http://stix.mitre.org/stix-1'

    ttp = fields.TypedField("TTP", TTP, multiple=True, key_name="ttps", listfunc=partial(IdrefDeprecatedList, type=TTP))
    kill_chains = fields.TypedField("Kill_Chains", KillChains)

    def __init__(self, ttps=None):
        super(TTPs, self).__init__()
        self.ttp = ttps
        self.kill_chains = KillChains()

    def add_ttp(self, ttp):
        self.ttp.append(ttp)



import unittest
from stix.core import STIXPackage
from mixbox.vendor.six import StringIO, text_type

from stix.test import EntityTestCase
from stix.extensions.malware.maec_4_1_malware import MAECInstance

try:
    import maec
    maec_present = True
except ImportError:
    maec_present = False


@unittest.skipIf(condition=maec_present is False, reason="These tests require the 'maec' library.")
class PythonMAECTests(EntityTestCase, unittest.TestCase):
    klass = MAECInstance

    _full_dict = {
        'xsi:type': 'stix-maec:MAEC4.1InstanceType',
        'maec': {
            'malware_subjects':
                [
                    {
                        'malware_instance_object_attributes': {
                            'id': 'maec-tst-obj-1',
                            'properties': {
                                'hashes':
                                    [
                                        {
                                            'simple_hash_value': '9d7006e30fdf15e9c8e03e62534b3a3e',
                                            'type': 'MD5'
                                        }
                                    ],
                                'xsi:type': 'FileObjectType'
                            }
                        }
                    }
                ]
        }
    }

    def test_add_name_type(self):
        maec_malware_instance = MAECInstance()
        maec_malware_instance.add_name("Poison Ivy Variant v4392-acc")
        maec_malware_instance.add_type("Remote Access Trojan")
        maec_xml = text_type(maec_malware_instance.to_xml())
        self.assertTrue("Poison Ivy Variant v4392-acc" in maec_xml)
        self.assertTrue("Remote Access Trojan" in maec_xml)


@unittest.skipIf(condition=maec_present is False, reason="These tests require the 'maec' library.")
class PythonMAECInPackageTests(unittest.TestCase):
    XML = StringIO(
        """
        <stix:STIX_Package
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xmlns:stix="http://stix.mitre.org/stix-1"
            xmlns:stixCommon="http://stix.mitre.org/common-1"
            xmlns:stixVocabs="http://stix.mitre.org/default_vocabularies-1"
            xmlns:ttp="http://stix.mitre.org/TTP-1"
            xmlns:example="http://example.com"
            xsi:schemaLocation="
            http://stix.mitre.org/TTP-1 http://stix.mitre.org/XMLSchema/ttp/1.2/ttp.xsd
            http://stix.mitre.org/common-1 http://stix.mitre.org/XMLSchema/common/1.2/stix_common.xsd
            http://stix.mitre.org/default_vocabularies-1 http://stix.mitre.org/XMLSchema/default_vocabularies/1.2.0/stix_default_vocabularies.xsd
            http://stix.mitre.org/stix-1 http://stix.mitre.org/XMLSchema/core/1.2/stix_core.xsd"
            id="example:Package-2b8fb66f-b6b3-4d40-865a-33e4a5ee1246"
            version="1.2"
            >
          <stix:TTPs>
            <stix:TTP xsi:type="ttp:TTPType" id="example:ttp-7d9fe1f7-429d-077e-db51-92c70b8da45a">
              <ttp:Title>Poison Ivy Variant v4392-acc</ttp:Title>
              <ttp:Behavior>
                <ttp:Malware>
                  <ttp:Malware_Instance>
                    <ttp:Type xsi:type="stixVocabs:MalwareTypeVocab-1.0">Remote Access Trojan</ttp:Type>
                    <ttp:Name>Poison Ivy Variant v4392-acc</ttp:Name>
                  </ttp:Malware_Instance>
                </ttp:Malware>
              </ttp:Behavior>
            </stix:TTP>
          </stix:TTPs>
        </stix:STIX_Package>
        """
    )
    XML_MAEC = StringIO(
        """
        <stix:STIX_Package
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xmlns:stix="http://stix.mitre.org/stix-1"
            xmlns:stixCommon="http://stix.mitre.org/common-1"
            xmlns:stixVocabs="http://stix.mitre.org/default_vocabularies-1"
            xmlns:ttp="http://stix.mitre.org/TTP-1"
            xmlns:stix-maec="http://stix.mitre.org/extensions/Malware#MAEC4.1-1"
            xmlns:maecPackage="http://maec.mitre.org/XMLSchema/maec-package-2"
            xmlns:example="http://example.com"
            xsi:schemaLocation="
            http://stix.mitre.org/TTP-1 http://stix.mitre.org/XMLSchema/ttp/1.2/ttp.xsd
            http://stix.mitre.org/common-1 http://stix.mitre.org/XMLSchema/common/1.2/stix_common.xsd
            http://stix.mitre.org/default_vocabularies-1 http://stix.mitre.org/XMLSchema/default_vocabularies/1.2.0/stix_default_vocabularies.xsd
            http://stix.mitre.org/stix-1 http://stix.mitre.org/XMLSchema/core/1.2/stix_core.xsd
            http://stix.mitre.org/extensions/Malware#MAEC4.1-1 http://stix.mitre.org/XMLSchema/extensions/malware/maec_4.1/1.0/maec_4.1_malware.xsd
            http://maec.mitre.org/XMLSchema/maec-package-2 http://maec.mitre.org/language/version4.1/maec_package_schema.xsd"
            id="example:Package-2b8fb66f-b6b3-4d40-865a-33e4a5ee1246"
            version="1.2"
            >
          <stix:TTPs>
            <stix:TTP xsi:type="ttp:TTPType" id="example:ttp-7d9fe1f7-429d-077e-db51-92c70b8da45a">
              <ttp:Title>Poison Ivy Variant v4392-acc</ttp:Title>
              <ttp:Behavior>
                <ttp:Malware>
                  <ttp:Malware_Instance xsi:type="stix-maec:MAEC4.1InstanceType">
                    <ttp:Type xsi:type="stixVocabs:MalwareTypeVocab-1.0">Remote Access Trojan</ttp:Type>
                    <ttp:Name>Poison Ivy Variant v4392-acc</ttp:Name>
                    <stix-maec:MAEC id="example:package-2fb96bef-1b11-436e-af4a-15588ac3198b" schema_version="2.1">
                      <maecPackage:Malware_Subjects>
                        <maecPackage:Malware_Subject id="example:Subject-57cd4839-436e-1b11-af4a-15588ac3198b">
                          <maecPackage:Malware_Instance_Object_Attributes>
                          </maecPackage:Malware_Instance_Object_Attributes>
                        </maecPackage:Malware_Subject>
                      </maecPackage:Malware_Subjects>
                    </stix-maec:MAEC>
                  </ttp:Malware_Instance>
                </ttp:Malware>
              </ttp:Behavior>
            </stix:TTP>
          </stix:TTPs>
        </stix:STIX_Package>
        """
    )

    def test_parse_malware(self):
        """Test parsing a normal MalwareInstance from XML
        """
        stix_pkg = STIXPackage.from_xml(self.XML)
        mw = stix_pkg.ttps.ttp[0].behavior.malware_instances[0].to_dict()
        self.assertTrue('names' in mw)

    def test_parse_malware_maec(self):
        """Test parsing a MaecInstance from XML
        """
        stix_pkg = STIXPackage.from_xml(self.XML_MAEC)
        mw = stix_pkg.ttps.ttp[0].behavior.malware_instances[0].to_dict()
        self.assertTrue('names' in mw)


@unittest.skipIf(condition=maec_present is True, reason="These tests require the 'maec' library to be missing.")
class PythonMAECNotInstalledTest(unittest.TestCase):

    def test_parsing_maec_fails(self):
        try:
            STIXPackage.from_xml(PythonMAECInPackageTests.XML_MAEC)
        except ImportError as e:
            self.assertTrue(all(x in str(e) for x in ("No module named", "maec")))

    def test_handling_maec_object_fails(self):
        try:
            MAECInstance().from_dict(PythonMAECTests._full_dict)
        except ImportError as e:
            self.assertTrue(all(x in str(e) for x in ("No module named", "maec")))

    def test_setting_maec_property_fails(self):
        try:
            m = MAECInstance()
            m.maec = "foo"
        except ImportError as e:
            self.assertTrue(all(x in str(e) for x in ("No module named", "maec")))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from io import open  # Allow `encoding` kwarg on Python 2.7
from os.path import abspath, dirname, join


from setuptools import setup, find_packages

BASE_DIR = dirname(abspath(__file__))
VERSION_FILE = join(BASE_DIR, 'stix', 'version.py')


def get_version():
    with open(VERSION_FILE) as f:
        for line in f.readlines():
            if line.startswith("__version__"):
                version = line.split()[-1].strip('"')
                return version
        raise AttributeError("Package does not have a __version__")


def get_long_description():
    with open('README.rst', encoding='utf-8') as f:
        return f.read()


install_requires = [
    'lxml>=2.2.3 ; python_version == "2.7" or python_version >= "3.5"',
    'lxml>=2.2.3,<4.4.0 ; python_version > "2.7" and python_version < "3.5"',
    'mixbox>=1.0.2',
    'cybox>=2.1.0.13,<2.1.1.0',
    'python-dateutil',
]


setup(
    name="stix",
    version=get_version(),
    author="STIX Project, MITRE Corporation",
    author_email="stix@mitre.org",
    description="An API for parsing and generating STIX content.",
    long_description=get_long_description(),
    url="https://stixproject.github.io/",
    packages=find_packages(),
    install_requires=install_requires,
    license="BSD",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    project_urls={
        'Documentation': 'https://stix.readthedocs.io/',
        'Source Code': 'https://github.com/STIXProject/python-stix/',
        'Bug Tracker': 'https://github.com/STIXProject/python-stix/issues/',
    },
)
